<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              factureaza.ro
 * @since             1.4.3
 * @package           F4
 *
 * @wordpress-plugin
 * Plugin Name:       f4
 * Plugin URI:        https://factureaza.ro
 * Description:       factureaza plugin integrates WooCommerce with factureaza.ro invoicing app.
 * Version:           1.4.3
 * Author:            factureaza.ro
 * Author URI:        factureaza.ro
 * Requires PHP:      7.2
 * Tested up to:      6.0
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       f4
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Currently plugin version.
 * Start at version 1.4.3 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'FAC_WOO_F4_VERSION', '1.4.3' );

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-f4-activator.php
 */
function fac_woo_f4_activate_f4() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-f4-activator.php';
	F4_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-f4-deactivator.php
 */
function fac_woo_f4_deactivate_f4() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-f4-deactivator.php';
	F4_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'fac_woo_f4_activate_f4' );
register_deactivation_hook( __FILE__, 'fac_woo_f4_deactivate_f4' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-f4.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.4.3
 */

function fac_woo_f4_db_secret() {
	global $wpdb;
    $f4 = "REWSAKSDNSFFQYUFHALNOMKNEPZIOPLW"; 
    $charset_collate = $wpdb->get_charset_collate();

    $sql = "CREATE TABLE $f4 (
		id mediumint(9) NOT NULL AUTO_INCREMENT,
		sk varchar(255) NOT NULL,
		s4 varchar(255) NOT NULL,
		UNIQUE KEY id (id)
	) $charset_collate;";

	$results = $wpdb->get_results( "SELECT * FROM " . $f4 . " ORDER BY id ASC LIMIT 1", OBJECT );
	if(empty($results[0])) {
		$sk = substr( str_shuffle(md5(time())), 0, 32 );
		$s4 = substr( str_shuffle(md5(time())), 0, 32 );

		$wpdb->insert( 
			$f4, 
			array( 
				'sk' => $sk,
				's4' => $s4
			) 
		);
	}

	require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
	dbDelta( $sql );
}

function fac_woo_f4_db_api_key() {
	global $wpdb;
    $f4 = $wpdb->prefix . "wc_f4_api_key"; 
    $charset_collate = $wpdb->get_charset_collate();

    $sql = "CREATE TABLE $f4 (
		id mediumint(9) NOT NULL AUTO_INCREMENT,
		api_key varchar(255) NOT NULL,
		encryption_type varchar(255),
		UNIQUE KEY id (id)
	) $charset_collate;";

    require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
	dbDelta( $sql );

	// add index on api_key
	$sql = "CREATE INDEX api_key
	ON $f4 (api_key) $charset_collate;"; 	

	require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
	dbDelta( $sql );
}

function fac_woo_f4_db_invoicing() {
	global $wpdb;
    $f4 = $wpdb->prefix . "wc_f4_invoicing"; 
    $charset_collate = $wpdb->get_charset_collate();

    $sql = "CREATE TABLE $f4 (
		id mediumint(9) NOT NULL AUTO_INCREMENT,
		invoice_id varchar(255) NOT NULL,
		order_id varchar(255) NOT NULL,
		invoice_type varchar(255) NOT NULL,
		UNIQUE KEY id (id)
	) $charset_collate;";

    require_once( ABSPATH . 'wp-admin/includes/upgrade.php' ); 
	dbDelta( $sql );
	
	// add index on invoice_id
	$sql = "CREATE INDEX invoice_id
	ON $f4 (invoice_id) $charset_collate;"; 	

	require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
	dbDelta( $sql );
	
	// add index on order_id
	$sql = "CREATE INDEX order_id
	ON $f4 (order_id) $charset_collate;"; 	

	require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
	dbDelta( $sql );
}

function fac_woo_f4_db_settings() {
	global $wpdb;
    $f4 = $wpdb->prefix . "wc_f4_settings"; 
    $charset_collate = $wpdb->get_charset_collate();

    $sql = "CREATE TABLE $f4 (
		id mediumint(9) NOT NULL AUTO_INCREMENT,
		default_country varchar(255),
		custom_series_prefix varchar(255),
		custom_shipping_vat varchar(255),
		custom_series_suffix varchar(255),
		multiple_currencies mediumint(9),
		proforma_invoices mediumint(9),
		custom_redirect varchar(255),
		auto_invoicing mediumint(9),
		auto_email mediumint(9),
		invoicing_without_client_address mediumint(9),
		vat_on_transport mediumint(9),
		tax_rate_shipping_included_vat mediumint(9),
		document_state varchar(255),
		document_position_unit varchar(255),
		shipping_tax_name varchar(255),
		company_label_name varchar(255),
		stripe_integration_workflow mediumint(9),
		stripe_shipping_tax_value varchar(255),
		stripe_shipping_tax_value_includes_vat mediumint(9),
		stripe_document_position_unit_name varchar(255),
		stripe_shipping_description varchar(255),
		UNIQUE KEY id (id)
	) $charset_collate;";

    require_once( ABSPATH . 'wp-admin/includes/upgrade.php' ); 
	dbDelta( $sql );
}

function fac_woo_f4_db_exchange_rates() {
	global $wpdb;
    $f4 = $wpdb->prefix . "wc_f4_exchange_rates"; 
    $charset_collate = $wpdb->get_charset_collate();

    $sql = "CREATE TABLE $f4 (
		id mediumint(9) NOT NULL AUTO_INCREMENT,
		order_date varchar(255),
		exchange_rate varchar(255),
		currencies varchar(255),
		UNIQUE KEY id (id)
	) $charset_collate;";

    require_once( ABSPATH . 'wp-admin/includes/upgrade.php' ); 
	dbDelta( $sql );
}

function fac_woo_f4_run_f4() {
	$plugin = new F4();
	$plugin->run();

	fac_woo_f4_db_secret();
	fac_woo_f4_db_api_key();
	fac_woo_f4_db_invoicing();
	fac_woo_f4_db_settings();
	fac_woo_f4_db_exchange_rates();
}
fac_woo_f4_run_f4();

function woocommerce_absent_error() {
    wp_die(__('<strong>factureaza plugin</strong> needs Woocommerce to be installed. <a href="#" onclick="window.history.back();">Back</a>.', 'factureaza'));
}

function woocommerce_incompatible_error() {
	wp_die(__('<strong>factureaza plugin</strong> needs Woocommerce version higher than 3.0.0 and WordPress version higher than 4.7.0 to be installed. <a href="#" onclick="window.history.back();">Back</a>.', 'factureaza'));
}

function woocommerce_presnece() {
	if ( class_exists( 'WooCommerce' ) ) {
		return true;
	} else {
		return false;
	}
}

function check_f4_compatibility() {
    $min_wp_ver = '4.7.0';
    $min_woo_ver = '3.0.0';

    // check woocommerce version to be higher than 4.7
    if ( version_compare( $GLOBALS['wp_version'], $min_wp_ver, '<' ) 
    ) {
        return false;
    }
    if ( class_exists( 'WooCommerce' ) ) {
        global $woocommerce;
        if( version_compare( $woocommerce->version, $min_woo_ver, "<" ) ) {
            return false;
        }
    }
    else {
        return false;
    }

    return true;
}

?>
